#!/usr/bin/env python
# *-* coding: utf-8 *-*
#
# setup.py
# Copyright (C) 2011-2012 Davide Depau <david.dep.1996@gmail.com> and Vittorio Cagnetta <vaisarger@gmail.com>
# 
# Bash!Karaoke GTK, Bash!Karaoke and EasyBashGUI are free software:
# you can redistribute them and/or modify them
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Bash!Karaoke GTK, Bash!Karaoke and EasyBashGUI are distributed
# in the hope that they will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with these programs.    If not, see <http://www.gnu.org/licenses/>.

try:
    import os, sys, argparse, shutil, subprocess, urllib, tarfile, platform
except ImportError, errorr:
    sys.stderr.write("Import Error: " + str(errorr) + "\nTry to run \"sudo apt-get install python-argparse\"")
    
try:
    import apt
    debian = True
except:
    debian = False

parser = argparse.ArgumentParser(description="Installs, removes or upgrades Bash!Karaoke GTK in your computer", prog="sudo python setup.py")
subparser = parser.add_subparsers(title="Options")
p_install = subparser.add_parser("install", help="Installs/Upgrades Bash!Karaoke GTK")
p_remove = subparser.add_parser("remove", help="Removes Bash!Karaoke GTK")

instdir = "/usr/share/bashkaraoke-gtk"

def install(bkdir, cwd, icons):
    if platform.dist()[0] in ("Ubuntu") and float(platform.dist()[1]) <= 10.04:
        error("Bash!Karaoke GTK is not compatible with Ubuntu 10.04.", 1, False)
        error("Please upgrade to a newer version of Ubuntu.", 1, True)
    else: pass
    # Install dependencies
    info("Checking dependencies...")
    devnull = open("/dev/null", "a")
    if debian:
        cache = apt.Cache()
        python = cache["python2.7"]
        pyminimal = cache["python2.7-minimal"]
        gtk3 = cache["gir1.2-gtk-3.0"]
        gtk2 = cache["gir1.2-gtk-2.0"]
        glib = cache["gir1.2-glib-2.0"]
        gst = cache["python-gst0.10"]
        gdkpixbuf = cache["gir1.2-gdkpixbuf-2.0"]
        timidity = cache["timidity"]
        bash = cache["bash"]
        
        if not python.is_installed or not pyminimal.is_installed or not gtk3.is_installed or not gtk2.is_installed or not glib.is_installed or not gst.is_installed or not gst.is_installed or not gdkpixbuf.is_installed or not timidity.is_installed or not bash.is_installed:
            missing = True
        else:
            missing = False
            
        if missing:
            question = raw_input("\033[0;33m[WARNING]\033[0m Some package missing; do you want to install them? (Y/n) ")
            if not question.startswith("n") and not question.startswith("N"):
                if not python.is_installed:
                    python.mark_install()
                else:
                    if python.is_upgradable:
                        python.mark_upgrade()
                    else: pass
                if not pyminimal.is_installed:
                    pyminimal.mark_install()
                else:
                    if pyminimal.is_upgradable:
                        pyminimal.mark_upgrade()
                    else: pass
                if not gtk3.is_installed:
                    gtk3.mark_install()
                else:
                    if gtk3.is_upgradable:
                        gtk3.mark_upgrade()
                    else: pass
                if not gtk2.is_installed:
                    gtk2.mark_install()
                else:
                    if gtk2.is_upgradable:
                        gtk2.mark_upgrade()
                    else: pass
                if not glib.is_installed:
                    glib.mark_install()
                else:
                    if glib.is_upgradable:
                        glib.mark_upgrade()
                    else: pass
                if not gst.is_installed:
                    gst.mark_install()
                else:
                    if gst.is_upgradable:
                        gst.mark_upgrade()
                    else: pass
                if not gdkpixbuf.is_installed:
                    gdkpixbuf.mark_install()
                else:
                    if gdkpixbuf.is_upgradable:
                        gdkpixbuf.mark_upgrade()
                    else: pass
                if not timidity.is_installed:
                    timidity.mark_install()
                else:
                    if timidity.is_upgradable:
                        timidity.mark_upgrade()
                    else: pass
                if not bash.is_installed:
                    bash.mark_install()
                else:
                    if bash.is_upgradable:
                        bash.mark_upgrade()
                    else: pass
                info("Installing packages...")
                try:
                    cache.commit()
                except apt.cache.LockFailedException, a:
                    error(str(a), 1, False)
                    error("An error occurred while installing packages", 1, True)
            else:
                info("Run the following command to install the needed packages\n")
                print "sudo apt-get install python2.7 python2.7-minimal gir1.2-gtk-3.0 gir1.2-gtk-2.0 gir1.2-glib-2.0 gir1.2-gdkpixbuf-2.0 python-gst0.10 timidity bash\n"
                
                info("In most terminals you can select some text, click it with the middle")
                info("button of the mouse (usually the wheel, or if you use a touchpad, both")
                info("the keys of the touchpad pressed together) and press Return (Enter) to run it.\n")
                sys.exit(0)
                
        else: pass
        
    else:
        missing = []
        a = subprocess.Popen(["which", "python2.7"], stdout=devnull)
        b = subprocess.Popen(["which", "timidity"], stdout=devnull)
        c = subprocess.Popen(["which", "bash"], stdout=devnull)
        a.wait()
        if int(a.poll()) > 0:
            missing = missing + ["python2.7"]
        b.wait()
        if int(b.poll()) > 0:
            missing = missing + ["timidity"]
        c.wait()
        if int(c.poll()) > 0:
            missing = missing + ["bash"]
        if not os.path.exists("/usr/lib/girepository-1.0/Gtk-3.0.typelib"):
            missing = missing + ["gir1.2-gtk-3.0 (gi.repository.Gtk, gi.repository.Gdk)"]
        if not os.path.exists("/usr/lib/girepository-1.0/Gtk-2.0.typelib"):
            missing = missing + ["gir1.2-gtk-2.0 (gi.repository.Gtk, gi.repository.Gdk)"]
        if not os.path.exists("/usr/lib/girepository-1.0/GObject-2.0.typelib"):
            missing = missing + ["gir1.2-glib-2.0 (gi.repository.GObject)"]
        if not os.path.exists("/usr/lib/girepository-1.0/GdkPixbuf-2.0.typelib"):
            missing = missing + ["gir1.2-gdkpixbuf-2.0 (gi.repository.GdkPixbuf)"]
        if not os.path.exists("/usr/lib/python2.7/dist-packages/pygst.py"):
            missing = missing + ["python-gst0.10 (gst, pygst)"]
        if not os.path.exists("/usr/lib/python2.7/os.py"):
            missing = missing + ["python2.7-minimal (python built-in modules)"]
            
        if missing != []:
            error("Some packages missing:", 1, False)
            warning(str(missing), 1, False)
            print "\n"
            warning("Try to install them and retry.", 1, True)
        else: pass
        
    ebg = subprocess.Popen(["which", "easybashgui"], stdout=devnull)
    bk = subprocess.Popen(["which", "bashkaraoke"], stdout=devnull)
    
    ebg.wait()
    if int(ebg.poll()) > 0:
        q2 = raw_input("\033[0;33m[WARNING]\033[0m EasyBashGUI is missing; do you want to install it? (Y/n) ")
        if not q2.startswith("n") and not q2.startswith("N"):
            try:
                os.mkdir("/tmp/ebg")
            except: pass
            cwd = os.getcwd()
            sys.stdout.write("\033[0;36m[PROGRESS] \033[0mDownloading EasyBashGUI...")
            sys.stdout.flush()
            ebg_last = urllib.urlopen("http://bashkaraoke.altervista.org/files/ebg-last.tar.gz")
            tard1 = urllib.urlopen("http://bashkaraoke.altervista.org/files/ebgdir")
            tard2 = urllib.urlopen("http://bashkaraoke.altervista.org/files/ebgver")
            tmp1 = open("/tmp/ebg/ebg-last.tar.gz", 'w')
            tmp1.write(ebg_last.read())
            directory1 = str(tard1.read())
            ver = str(tard2.read())
            ebg_last.close()
            tmp1.close()
            tard1.close()
            tard2.close()
            sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mEasyBashGUI downloaded.                \n")
            sys.stdout.flush()
            if directory1.endswith("\n"):
                directory1 = directory1[:-1]
            if ver.endswith("\n"):
                ver = ver[:-1]
            os.chdir("/tmp/ebg/")
            sys.stdout.write("\033[0;36m[PROGRESS] \033[0mUnpacking EasyBashGUI...")
            sys.stdout.flush()
            tar = tarfile.open("/tmp/ebg/ebg-last.tar.gz")
            tar.extractall()
            tar.close()
            sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mEasyBashGUI unpacked.                \n")
            sys.stdout.flush()
            sys.stdout.write("\033[0;36m[PROGRESS] \033[0mInstalling EasyBashGUI...")
            sys.stdout.flush()
            shutil.copy("/tmp/ebg/" + directory1 + "/easybashgui_" + ver, "/usr/local/bin/easybashgui_" + ver)
            shutil.copy("/tmp/ebg/" + directory1 + "/Ok.xpm", "/usr/local/bin/Ok.xpm")
            shutil.copy("/tmp/ebg/" + directory1 + "/Attenzione.xpm", "/usr/local/bin/Attenzione.xpm")
            chmod = subprocess.Popen(["chmod", "755", "/usr/local/bin/easybashgui_" + ver], stdout=devnull)
            chmod.wait()
            try:
                os.remove("/usr/local/bin/easybashgui")
            except: pass
            os.symlink("/usr/local/bin/easybashgui_" + ver, "/usr/local/bin/easybashgui")
            chmod1 = subprocess.Popen(["chmod", "755", "/usr/local/bin/easybashgui"], stdout=devnull)
            chmod1.wait()
            sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mEasyBashGUI installed.                \n")
            sys.stdout.flush()
            os.chdir(cwd)
        else: pass
    
    bk.wait()
    if int(bk.poll()) > 0:
        q = raw_input("\033[0;33m[WARNING]\033[0m Bash!Karaoke is missing; do you want to install it? (Y/n) ")
        if not q.startswith("n") and not q.startswith("N"):
            try:
                os.mkdir("/tmp/bk")
            except: pass
            cwd = os.getcwd()
            sys.stdout.write("\033[0;36m[PROGRESS] \033[0mDownloading Bash!Karaoke...")
            sys.stdout.flush()
            bk_last = urllib.urlopen("http://bashkaraoke.altervista.org/files/bk-last.tar.gz")
            tard = urllib.urlopen("http://bashkaraoke.altervista.org/files/bkdir")
            tmp = open("/tmp/bk/bk-last.tar.gz", 'w')
            tmp.write(bk_last.read())
            directory = str(tard.read())
            bk_last.close()
            tmp.close()
            tard.close()
            sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mBash!Karaoke downloaded.                \n")
            sys.stdout.flush()
            if directory.endswith("\n"):
                directory = directory[:-1]
            os.chdir("/tmp/bk/")
            sys.stdout.write("\033[0;36m[PROGRESS] \033[0mUnpacking Bash!Karaoke...")
            sys.stdout.flush()
            tar = tarfile.open("/tmp/bk/bk-last.tar.gz")
            tar.extractall()
            tar.close()
            sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mBash!Karaoke unpacked.                \n")
            sys.stdout.flush()
            sys.stdout.write("\033[0;36m[PROGRESS] \033[0mInstalling Bash!Karaoke...")
            sys.stdout.flush()
            os.chdir("/tmp/bk/" + directory + "/")
            install = subprocess.Popen(["bash", "bashkaraoke-INSTALL-UPGRADE"], stdout=devnull, stderr=devnull)
            install.wait()
            sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mBash!Karaoke installed.                \n")
            sys.stdout.flush()
            os.chdir(cwd)
        else: pass
        
    info("Dependencies installed.")
    # Install Bash!Karaoke GTK
    if not os.path.exists(bkdir):
        os.mkdir(bkdir)
    else: pass
    if not os.path.exists(bkdir + "/ui"):
        os.mkdir(bkdir + "/ui")
    else: pass
    sys.stdout.write("\033[0;36m[PROGRESS] \033[0mCoping files... 0%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/src/bashkaraoke-gtk.py", bkdir + "/bashkaraoke-gtk.py")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 8%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/src/intro.py", bkdir + "/intro.py")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 15%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/data/bash-karaoke.ui", bkdir + "/ui/bash-karaoke.ui")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 23%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/src/bashkaraoke-gtk.py", bkdir + "/bashkaraoke-gtk.py")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 31%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/src/gtk3_oxygen.bash", bkdir + "/gtk3_oxygen.bash")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 38%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/src/oxyquestion.bash", bkdir + "/oxyquestion.bash")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 46%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/src/ohno.bash", bkdir + "/ohno.bash")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 54%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/src/uff.bash", bkdir + "/uff.bash")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 62%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/src/oxynodebian.bash", bkdir + "/oxynodebian.bash")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 69%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/data/bannerino.png", bkdir + "/ui/bannerino.png")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 77%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/data/cc.png", bkdir + "/ui/cc.png")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 85%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/data/intro1080i.ogg", bkdir + "/ui/intro1080i.ogg")
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mCoping files... 92%")
    sys.stdout.flush()
    try: shutil.copy(cwd + "/data/intro720p.ogg", bkdir + "/ui/intro720p.ogg")
    except: pass
    try:
        shutil.copy(cwd + "/src/bashkaraoke.desktop", "/usr/share/applications/bashkaraoke.desktop")
        chmod2 = subprocess.Popen(["chmod", "+x", "/usr/share/applications/bashkaraoke.desktop"], stdout=devnull)
        chmod2.wait()
    except: pass
    sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mFiles copied                \n")
    sys.stdout.flush()
    if os.path.exists("/usr/bin/bashkaraoke-gtk"):
        try: os.remove("/usr/bin/bashkaraoke-gtk")
        except: pass
    else: pass
    os.symlink(bkdir + "/bashkaraoke-gtk.py", "/usr/bin/bashkaraoke-gtk")

    if icons == True:
        sys.stdout.write("\033[0;36m[PROGRESS] \033[0mInstalling icons... 0%")
        sys.stdout.flush()
        a = subprocess.Popen("xdg-icon-resource install --novendor --context apps --mode system --size 14 " + cwd + "/data/bk14.png bashkaraoke", shell=True)
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mInstalling icons... 12%")
        sys.stdout.flush()
        b = subprocess.Popen("xdg-icon-resource install --novendor --context apps --mode system --size 16 " + cwd + "/data/bk16.png bashkaraoke", shell=True)
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mInstalling icons... 25%")
        sys.stdout.flush()
        c = subprocess.Popen("xdg-icon-resource install --novendor --context apps --mode system --size 22 " + cwd + "/data/bk22.png bashkaraoke", shell=True)
        a.wait()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mInstalling icons... 37%")
        sys.stdout.flush()
        d = subprocess.Popen("xdg-icon-resource install --novendor --context apps --mode system --size 32 " + cwd + "/data/bk32.png bashkaraoke", shell=True)
        b.wait()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mInstalling icons... 50%")
        sys.stdout.flush()
        e = subprocess.Popen("xdg-icon-resource install --novendor --context apps --mode system --size 48 " + cwd + "/data/bk48.png bashkaraoke", shell=True)
        c.wait()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mInstalling icons... 62%")
        sys.stdout.flush()
        f = subprocess.Popen("xdg-icon-resource install --novendor --context apps --mode system --size 64 " + cwd + "/data/bk64.png bashkaraoke", shell=True)
        d.wait()
        sys.stdout.flush()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mInstalling icons... 75%")
        sys.stdout.flush()
        g = subprocess.Popen("xdg-icon-resource install --novendor --context apps --mode system --size 128 " + cwd + "/data/bk128.png bashkaraoke", shell=True)
        e.wait()
        sys.stdout.flush()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mInstalling icons... 87%")
        sys.stdout.flush()
        last = subprocess.Popen("xdg-icon-resource install --novendor --context apps --mode system --size 192 " + cwd + "/data/bk192.png bashkaraoke", shell=True)
        f.wait()
        g.wait()
        last.wait()
        sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mIcons installed                \n")
        sys.stdout.flush()
    else: pass
    devnull.close()
    success("\r                            \n\033[0;32m[SUCCESS] \033[0mBash!Karaoke GTK succesfully installed!")
    success("Run «bashkaraoke» and set «GTK» in window settings, or simply run «bashkaraoke-gtk»")
    sys.exit(0)

def remove(bkdir, icons):
    confirm = raw_input("\033[0;33m[WARNING]\033[0m Bash!Karaoke may not work if you remove Bash!Karaoke GTK.\n\033[0;33m[WARNING]\033[0m Are you sure you want to continue? (Y/n) \033[0m ")
    if confirm.startswith("y"):
        pass
    else:
        sys.exit("Canceled")
    sys.stdout.write("\n\033[0;36m[PROGRESS] \033[0mRemoving files... 0%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/bashkaraoke-gtk.py")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 13%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/gtk3_oxygen.bash")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 20%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/ohno.bash")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 27%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/oxyquestion.bash")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 33%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/oxynodebian.bash")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 40%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/uff.bash")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 47%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/intro.py")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 53%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/ui/bash-karaoke.ui")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 60%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/ui/bannerino.png")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 67%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/ui/cc.png")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 73%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/ui/intro1080i.ogg")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 80%")
    sys.stdout.flush()
    try:
        os.remove(bkdir + "/ui/intro720p.ogg")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 86%")
    sys.stdout.flush()
    try:
        os.remove("/usr/bin/bashkaraoke-gtk")
    except OSError: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 93%")
    sys.stdout.flush()
    try:
        shutil.rmtree(bkdir + "/ui/", ignore_errors=True)
    except: pass
    sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving files... 100%")
    sys.stdout.flush()
    try:
        shutil.rmtree(bkdir, ignore_errors=True)
    except: pass
    sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mFiles removed              \n")
    sys.stdout.flush()

    if icons == True:
        sys.stdout.write("\033[0;36m[PROGRESS] \033[0mRemoving icons... 0%")
        sys.stdout.flush()
        a = subprocess.Popen("xdg-icon-resource uninstall --context apps --mode system --size 14 bashkaraoke", shell=True)
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving icons... 12%")
        sys.stdout.flush()
        b = subprocess.Popen("xdg-icon-resource uninstall --context apps --mode system --size 16 bashkaraoke", shell=True)
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving icons... 25%")
        sys.stdout.flush()
        c = subprocess.Popen("xdg-icon-resource uninstall --context apps --mode system --size 22 bashkaraoke", shell=True)
        a.wait()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving icons... 37%")
        sys.stdout.flush()
        d = subprocess.Popen("xdg-icon-resource uninstall --context apps --mode system --size 32 bashkaraoke", shell=True)
        b.wait()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving icons... 50%")
        sys.stdout.flush()
        e = subprocess.Popen("xdg-icon-resource uninstall --context apps --mode system --size 48 bashkaraoke", shell=True)
        c.wait()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving icons... 62%")
        sys.stdout.flush()
        f = subprocess.Popen("xdg-icon-resource uninstall --context apps --mode system --size 64 bashkaraoke", shell=True)
        d.wait()
        sys.stdout.flush()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving icons... 75%")
        sys.stdout.flush()
        g = subprocess.Popen("xdg-icon-resource uninstall --context apps --mode system --size 128 bashkaraoke", shell=True)
        e.wait()
        sys.stdout.flush()
        sys.stdout.write("\r\033[0;36m[PROGRESS] \033[0mRemoving icons... 87%")
        sys.stdout.flush()
        last = subprocess.Popen("xdg-icon-resource uninstall --context apps --mode system --size 192 bashkaraoke", shell=True)
        f.wait()
        g.wait()
        last.wait()
        sys.stdout.write("\r\033[0;32m[SUCCESS] \033[0mIcons removed                \n\n")
        sys.stdout.flush()
    else: pass

    success("Bash!Karaoke GTK removed succesfully; Bash!Karaoke may not work well")
    sys.exit(0)

def error(emessage, estatus, close):
    sys.stderr.write("\033[0;31m[ERROR] \033[0m" + emessage + "\n")
    if close == True:
        sys.stderr.write("\033[0;31m[ERROR] \033[0m" + "Bash!Karaoke GTK not installed/removed correctly\n")
        sys.exit(estatus)
    else: pass

def warning(wmessage):
    sys.stderr.write("\033[0;33m[WARNING] \033[0m" + wmessage)

def success(smessage):
    print "\033[0;32m[SUCCESS] \033[0m" + smessage

def info(imessage):
    print "\033[0;36m[INFO] \033[0m" + imessage


#if os.access('/', os.W_OK):
if not os.access('/', os.W_OK):
    error("You need to get root permissions to install Bash!Karaoke.", 1, False)
    error("Run «sudo python setup.py install/remove»", 1, True)  
else: pass

if "-h" in sys.argv:
    print "usage: sudo python setup.py [-h] {install,remove}\n\nInstalls, removes or upgrades Bash!Karaoke GTK in your computer\n\noptional arguments:\n  -h, --help        show this help message and exit\n\nOptions:\n  {install,remove}\n    install [-n]     Installs/Upgrades Bash!Karaoke GTK\n     -n, --no-icons  Don't install the icons\n    remove  [-i]     Removes Bash!Karaoke GTK\n     -i  --icons     Removes the icons from the computer"
    sys.exit()
elif "--help" in sys.argv:
    print "usage: sudo python setup.py [-h] {install,remove}\n\nInstalls, removes or upgrades Bash!Karaoke GTK in your computer\n\noptional arguments:\n  -h, --help        show this help message and exit\n\nOptions:\n  {install,remove}\n    install [-n]     Installs/Upgrades Bash!Karaoke GTK\n     -n, --no-icons  Don't install the icons\n    remove  [-i]     Removes Bash!Karaoke GTK\n     -i  --icons     Removes the icons from the computer"
    sys.exit()
elif "install" in sys.argv and "remove" in sys.argv:
    error("You cannot install and Bash!Karaoke GTK and, at the same time,  remove it!\n", 1, False)
    print "usage: sudo python setup.py [-h] {install,remove}\n\nInstalls, removes or upgrades Bash!Karaoke GTK in your computer\n\noptional arguments:\n  -h, --help        show this help message and exit\n\nOptions:\n  {install,remove}\n    install [-n]     Installs/Upgrades Bash!Karaoke GTK\n     -n, --no-icons  Don't install the icons\n    remove  [-i]     Removes Bash!Karaoke GTK\n     -i  --icons     Removes the icons from the computer"
    sys.exit()
elif "install" in sys.argv and "-n" in sys.argv:
    try:
        install(instdir, os.getcwd(), False)
    except KeyboardInterrupt:
        error("Canceled", 1, True)
elif "install" in sys.argv and "--no-icons" in sys.argv:
    try:
        install(instdir, os.getcwd(), False)
    except KeyboardInterrupt:
        error("Canceled", 1, True)
elif "install" in sys.argv and not "-n" in sys.argv and not "--no-icons" in sys.argv:
    try:
        install(instdir, os.getcwd(), True)
    except KeyboardInterrupt:
        error("Canceled", 1, True)
elif "remove" in sys.argv and "-i" in sys.argv:
    try:
        remove(instdir, True)
    except KeyboardInterrupt:
        error("Canceled", 1, True)
elif "remove" in sys.argv and "--icons" in sys.argv:
    try:
        remove(instdir, True)
    except KeyboardInterrupt:
        error("Canceled", 1, True)
elif "remove" in sys.argv and not "-i" in sys.argv and not "--icons" in sys.argv:
    try:
        remove(instdir, False)
    except KeyboardInterrupt:
        error("Canceled", 1, True)
else:
    print "usage: sudo python setup.py [-h] {install,remove}\n\nInstalls, removes or upgrades Bash!Karaoke GTK in your computer\n\noptional arguments:\n  -h, --help        show this help message and exit\n\nOptions:\n  {install,remove}\n    install [-n]     Installs/Upgrades Bash!Karaoke GTK\n     -n, --no-icons  Don't install the icons\n    remove  [-i]     Removes Bash!Karaoke GTK\n     -i  --icons     Removes the icons from the computer"
    sys.exit()
