#!/usr/bin/env python
# *-* coding: utf-8 *-*
#
# bashkaraoke-gtk.py
# Copyright (C) Davide Depau 2011-2012 <david.dep.1996@gmail.com>
# 
# Bash!Karaoke GTK is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Bash!Karaoke GTK is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program.    If not, see <http://www.gnu.org/licenses/>.

app_name = "bashkaraoke-gtk"
app_version = "5.0.0"

# Import os, sys, time, user, subprocess, thread, platform
try:
    import os
    import sys
    import time
    import user
    import subprocess
    import thread
    import gettext
    gettext.bindtextdomain('bashkaraoke-gtk')
    gettext.textdomain('bashkaraoke-gtk')
    from gettext import gettext as _
except ImportError, f:
    l_importerror = _("Import Error: ")
    sys.stderr.write(l_importerror, str(f))
    sys.exit(2)

# Color stdout messages
def error(emessage, kind, estatus, close):
    l_error = _("ERROR")
    sys.stderr.write("\033[0;31m[" + time.asctime() + " " + l_error + "] \033[0m" + emessage + "\n")
    log(emessage, "e")
    l_stopped = _("Bash!Karaoke GTK crashed")
    l_notstart = _("Bash!Karaoke GTK didn't start")
    if close == True:
        if kind == "crash":
            sys.stderr.write("\033[0;31m[" + time.asctime() + " " + l_error + "] \033[0m" + l_stopped + "\n")
            log(l_stopped, "e")
        elif kind == "import":
            sys.stderr.write("\033[0;31m[" + time.asctime() + " " + l_error + "] \033[0m" + l_notstart + "\n")
            log(l_notstart, "e")
        log("", "stop")
        sys.exit(estatus)
    else:
        pass

def warning(wmessage):
    sys.stderr.write("\033[0;33m[" + time.asctime() + " " + _("WARNING") + "] \033[0m" + wmessage + "\n")
    log(wmessage, "w")

def success(smessage):
    if show_debug == True:
        print "\033[0;32m[" + time.asctime() + " " + _("SUCCESS") + "] \033[0m" + smessage
    log(smessage, "s")

def info(imessage):
    if show_debug == True:
        print "\033[0;36m[" + time.asctime() + " " + _("INFO") + "] \033[0m" + imessage
    log(imessage, "i")

def echo(message, color):
    if show_debug == True:
        print "\033[" + str(color) + "m" + str(message) + "\033[0m"
    log(message, "echo")

def debug(dmessage):
    if show_debug == True:
        print "[" + time.asctime() + " " + _("DEBUG") + "] " + dmessage
    else: pass
    log(dmessage, "d")

# Write log
def log(logtext, logtype):
    log = open(user.home + "/.bashkaraoke/.bashkaraoke-gtk.log", "a")
    if logtype == "i":
        log.write("[" + time.asctime() + " " + _("INFO") + "] " + str(logtext) + "\n")
        log.close()
    
    elif logtype == "e":
        log.write("[" + time.asctime() + " " + _("ERROR") + "] " + str(logtext) + "\n")
        log.close()

    elif logtype == "stop":
        log.write("\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! " + _("STOPPED") + " !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n!!!!!!!!!!!!!!!!!!!!!!!!!! " + time.asctime() + " !!!!!!!!!!!!!!!!!!!!!!!!!!!!\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n")    

    elif logtype == "w":
        log.write("[" + time.asctime() + " " + _("WARNING") + "] " + str(logtext) + "\n")
        log.close()
    
    elif logtype == "s":
        log.write("[" + time.asctime() + " " + _("SUCCESS") + "] " + str(logtext) + "\n")
        log.close()
    
    elif logtype == "d":
        log.write("[" + time.asctime() + " " + _("DEBUG") + "] " + str(logtext) + "\n")
        log.close()

    elif logtype == "echo":
        log.write(str(logtext) + "\n")
        log.close()
    
    elif logtype == "init":
        log.write("\n\n################################### " + _("STARTED") + " ####################################\n########################## " + time.asctime() + " ############################\n################################################################################\n\n")
        log.close()

    elif logtype == "close":
        log.write("\n################################### " + _("EXITED") + " #####################################\n########################## " + time.asctime() + " ############################\n################################################################################\n\n")


    elif logtype == "restart":
        log.write("\n################################## " + _("RESTARTED") + " ###################################\n########################## " + time.asctime() + " ############################\n################################################################################\n\n")


    else:
        log.write(str(logtext) + "\n")
        log.close()

l_loading = _("Loading...")

# Create needed files
def first_run():
    global l_loading
    if not os.path.exists(user.home + "/.bashkaraoke/"):
        if os.path.exists(user.home + "/.bash!karaoke"):
            os.symlink(user.home + "/.bash!karaoke", user.home + "/.bashkaraoke")
            success(_("Symbolic link from ~/.bash!karaoke/ to ~/.bashkaraoke/ created"))
        else:
            os.mkdir(user.home + "/.bashkaraoke/")
            success(_("Bash!Karaoke folder (~/.bashkaraoke/) crated"))
    else: pass
    if not os.path.exists(user.home + "/.bashkaraoke/.bashkaraoke.rc"):
        if os.path.exists(user.home + "/.bashkaraoke/.bash!karaoke.rc"):
            os.symlink(user.home + "/.bashkaraoke/.bash!karaoke.rc", user.home + "/.bashkaraoke/.bashkaraoke.rc")
            success(_("Symbolic link from ~/.bashkaraoke/.bash!karaoke.rc to ~/.bashkaraoke/.bashkaraoke.rc created"))
        else:
            rc = open(user.home + "/.bashkaraoke/.bashkaraoke.rc", "w")
            rc.write('export use_gsw="false"\nexport use_buc="false"\nexport use_gtk="true"\nexport supermode="auto"\nexport karaoke_window="true"\nexport levels_window="true"\nexport channels_window="false"\nexport spectrogram_window="false"\nexport rows=2\nexport columns=24\nexport font="-b&h-lucidatypewriter-medium-r-normal-sans-70-*-*-*-*-*-iso8859-1"\nexport background="#000000"\nexport foreground_1="#008000"\nexport foreground_2="#FF0000"\nexport soundfont="/usr/local/share/timidity/PC51f.sf2"\nexport extract_way="internal"\nexport error_correction_chars=0\nexport LANG="' + str(os.environ.get("LC_CTYPE")) + '"\nexport char_encoding="ISO-8859-1"\nexport predefvolume=100\nexport predefpitch=0\nexport predefspeed=100\nexport font_family="Ubuntu 40"\nexport show_debug=0\n')
            rc.close()
            success(_("Bash!Karaoke config file (~/.bashkaraoke/.bashkaraoke.rc) created"))
    else: pass
    if not os.path.exists(user.home + "/.bashkaraoke/.bashkaraoke-gtk.log"):
        logg = open(user.home + "/.bashkaraoke/.bashkaraoke-gtk.log", "w")
        logg.write("")
        logg.close()
        success(_("Bash!Karaoke GTK log file (~/.bashkaraoke/.bashkaraoke-gtk.log) created"))
    else: pass
    lyrr = open(user.home + "/.bashkaraoke/.lyrics", "w")
    lyrr.write('<span font="50" weight="light">' + l_loading + '</span>')
    lyrr.close()
    pidd = open(user.home + "/.bashkaraoke/.pid", "w")
    pidd.write('BK_OUT-NEVER-STARTED')
    pidd.close()
    timee = open(user.home + "/.bashkaraoke/.time", "w")
    timee.write('')
    timee.close()
    if not os.path.exists(user.home + "/.bashkaraoke/.version"):
        verss = open(user.home + "/.bashkaraoke/.version", "w")
        verss.write(str(app_version_float - 0.1))
        verss.close()
    else: pass
first_run()
log("", "init")

# Enable / disable debug mode
debugg = open(user.home + '/.bashkaraoke/.bashkaraoke.rc', 'r')
for line in debugg.readlines():
    if "show_debug" in line:
        if not line.endswith("\""):
            show_debugg = int(line.split("=", 1)[1])
        else:
            try:
                show_debugg = int(line.split("=", 1)[1][1:-2])
            except:
                warning(_("show_debug value not found. Restoring default value..."))
                show_debugg = 0
        show_debug = bool(show_debugg)
        debug(_("Debug mode:") + " " + str(show_debug))
        break
    else: pass
debugg.close()

# Import Gtk
# pids = str(subprocess.check_output("ps -U 0", shell=True))
try:
    from gi.repository import Gtk
except ImportError, b:
    error(_('Gtk not found. Run "sudo apt-get install gir1.2-gtk-3.0 gir1.2-gtk-2.0"'), "import", 2, False)
    error(_("Details:") + " " + str(b), "import", 2, True)
# Import Gdk
try:
        from gi.repository import Gdk
except ImportError, e:
        error(_("Import Error:") + " " + str(e), "import", 2, True)

# Import GObject
try:
        from gi.repository import GObject
except ImportError, c:
        error(_('GObject not found. Run "sudo apt-get install python-gobject'), "import", 2, False)
        error(_("Details:") + " " + str(c), "import", 2, True)
#You must edit the following lines if you wish to install Bash!Karaoke GTK
#in a different directory.
bkdir = "/usr/share/bashkaraoke-gtk"
UI = bkdir + "/ui/bash-karaoke.ui"

fullscreen = False
playing = False
e_dagli = 0
filename = str("")

class Main:
    def __init__(self):
        # Setup builder
        self.builder = Gtk.Builder()
        self.builder.add_from_file(UI)

        # Connect signal slots
        self.builder.connect_signals(self)

        # Get widgets
        self.about_authors = self.builder.get_object("authors_toggle")
        self.about_back = self.builder.get_object("about_back")
        self.about_license = self.builder.get_object("license_toggle")
        self.about_notebook = self.builder.get_object("about_notebook")
        self.about_video_button = self.builder.get_object("about_the_video_button")
        self.actionApri = self.builder.get_object("actionApri")
        self.add = self.builder.get_object("actionAdd")
        self.background_color = self.builder.get_object("bg-color")
        self.columns_adj = self.builder.get_object("columns-adj")
        self.debug_switch = self.builder.get_object("debug-mode")
        self.duration = self.builder.get_object("duration")
        self.ebg_expander = self.builder.get_object("ebg_expander")
        self.esci = self.builder.get_object("actionEsci")
        self.extr_external = self.builder.get_object("extr-external")
        self.extr_internal = self.builder.get_object("extr-internal")
        self.filefilter = self.builder.get_object("filefilter")
        self.font_button = self.builder.get_object("font-family")
        self.force_stop = self.builder.get_object("force-stop")
        self.fullscreen_image = self.builder.get_object("fullscreen_image")
        self.fullscreen = self.builder.get_object("happyFullscreen")
        self.help_sf = self.builder.get_object("help-soundfonts")
        self.lang_entry = self.builder.get_object("lang-entry")
        self.lyrics = self.builder.get_object("lyrics")
        self.main = self.builder.get_object("main")
        self.main_notebook = self.builder.get_object("main_notebook")
        self.menuabout = self.builder.get_object("actionAbout")
        self.menubar = self.builder.get_object("menubar")
        self.menuplay = self.builder.get_object("actionPlay")
        self.menustop = self.builder.get_object("actionStop")
        self.openbutton = self.builder.get_object("openbutton")
        self.openbar = self.builder.get_object("openbar")
        self.open = self.builder.get_object("actionApri")
        self.pitch_adj = self.builder.get_object("pitch_adj")
        self.playpause = self.builder.get_object("playpause")
        self.predefpitch = self.builder.get_object("predefpitch")
        self.predefspeed = self.builder.get_object("predefspeed")
        self.predefvolume = self.builder.get_object("predefvolume")
        self.prefs = self.builder.get_object("actionPreferenze")
        self.progressbar = self.builder.get_object("progressbar")
        self.progress = self.builder.get_object("progresso")
        self.restore_all = self.builder.get_object("restore-all")
        self.restore = self.builder.get_object("restore")
        self.rows_adj = self.builder.get_object("rows-adj")
        self.sep_entry = self.builder.get_object("sep-entry")
        self.sep = self.builder.get_object("yetanotherseparator")
        self.settings_cancel = self.builder.get_object("settings-cancel")
        self.settings_ok = self.builder.get_object("settings-ok")
        self.settings = self.builder.get_object("settings")
        self.sf2_chooser = self.builder.get_object("sf2-chooser")
        self.show_channels = self.builder.get_object("show-channels")
        self.show_levels = self.builder.get_object("show-levels")
        self.show_lyrics = self.builder.get_object("show-lyrics")
        self.show_spectrum = self.builder.get_object("show-spectrum")
        self.show_the_video = self.builder.get_object("show_the_video")
        self.speed_adj = self.builder.get_object("speed_adj")
        self.sung_color = self.builder.get_object("sung_color")
        self.time = self.builder.get_object("time")
        self.toggle_buc = self.builder.get_object("use_buc")
        self.toggle_gsw = self.builder.get_object("use_gsw")
        self.toggle_gtk = self.builder.get_object("use_gtk")
        self.tolerance_combo = self.builder.get_object("tolerance-combo")
        self.toolbar = self.builder.get_object("toolbar")
        self.tosing_color = self.builder.get_object("tosing_color")
        self.videoabout = self.builder.get_object("videoabout")
        self.video_authors = self.builder.get_object("authors_toggle1")
        self.video_back = self.builder.get_object("about_back1")
        self.video_bashkaraoke = self.builder.get_object("about_bashkaraoke_button")
        self.video_license = self.builder.get_object("license_toggle1")
        self.video_notebook = self.builder.get_object("video_notebook")
        self.volume_adj = self.builder.get_object("volume_adj")
        self.what_are_soundfonts = self.builder.get_object("what-are-soundfonts")
        self.win_expander = self.builder.get_object("win_expander")
        self.winmode_combo = self.builder.get_object("winmode-combo")
        self.openlabel = self.builder.get_object("openlabel")
        self.main_box = self.builder.get_object("main_box")
        self.separatorr = self.builder.get_object("separatorr")

        self.l_btm = _("Bash the microphone and break your music!")
        self.l_fullscreen = _("Fullscreen")
        global l_loading

        # Use GTK3 toolbars
#        self.main_box.reorder_child(self.toolbar, 0)
#        self.main_box.reorder_child(self.separatorr, 1)
#        self.main_box.reorder_child(self.openbutton, 1)
        self.toolbar.get_style_context().add_class("primary-toolbar")
#        self.openbar.get_style_context().add_class("primary-toolbar")

        self.fullscreen.set_stock_id(Gtk.STOCK_FULLSCREEN)
        self.fullscreen.set_label(self.l_fullscreen)


#        self.TARGET_TYPE_URI_LIST = 80
#        self.dnd_list = [ ( 'text/uri-list', 0, self.TARGET_TYPE_URI_LIST ) ]

#        self.main_box.drag_dest_set(Gtk.DestDefaults.ALL, self.dnd_list, Gdk.DragAction.COPY)

        # Show the video
        cane = open(user.home + "/.bashkaraoke/.version", "r")
        old_v = str(cane.read())
        cane.close()
        if int(old_v[0:1]) < int(app_version[0:1]):
            info(_("Welcome to Bash!Karaoke 5.0.0 !!! ;)"))
            info(_("You are about to see a video showing"))
            info(_("new Bash!Karaoke's features..."))
            gatto = open(user.home + "/.bashkaraoke/.version", "w")
            gatto.write(app_version)
            gatto.close()
            self.show_video()
        else: pass

        
        # Get settings
        self.get_settings_both()
        self.check_modes()
        self.show_settings()

        self.time.hide()
        self.duration.hide()
        self.progressbar.hide()
        
        # Show main window
        self.main.show_all()

        if "kde" in os.environ.get("DESKTOP_SESSION"):
            self.check_oxygen()
        else: pass
        
        self.mainsize = self.main.get_size()

        info(_("Bash!Karaoke GTK started happily"))

    def show_video(self):
        tappeto = subprocess.Popen("python " + bkdir + "/intro.py", shell=True)
        tenda = tappeto.wait()

    # Fullscreen
    def on_fullscreen_clicked(self, main):
        global fullscreen
        if not fullscreen:
            self.main.fullscreen()
            self.fullscreen.set_stock_id(Gtk.STOCK_LEAVE_FULLSCREEN)
            self.fullscreen.set_label(_("Exit fullscreen"))
            self.menubar.hide()
            fullscreen = True
        elif fullscreen:
            self.main.unfullscreen()
            self.fullscreen.set_stock_id(Gtk.STOCK_FULLSCREEN)
            self.fullscreen.set_label(self.l_fullscreen)
            self.menubar.show()
            fullscreen = False

    def help_soundfonts(self, help_sf):
        self.what_are_soundfonts.show()
        self.what_are_soundfonts.run()
        self.what_are_soundfonts.hide()

    # About dialog
    def on_about(self, actionAbout):
        self.main_notebook.set_current_page(2)

    def on_about_bashkaraoke(self, video_bashkaraoke):
        self.main_notebook.set_current_page(2)

    def about_the_video(self, videoabout):
        self.main_notebook.set_current_page(3)

    def on_about_the_video(self, about_video_button):
        self.main_notebook.set_current_page(3)

    def about_back(self, about_back):
        self.main_notebook.set_current_page(0)

    def video_back(self, video_back):
        self.main_notebook.set_current_page(0)

    def about_authors_toggle(self, about_authors):
        self.about_license.set_active(False)
        while Gtk.events_pending():
            Gtk.main_iteration()
        if self.about_authors.get_active():
            self.about_notebook.set_current_page(1)
        else:
            self.about_notebook.set_current_page(0)

    def on_about_license(self, about_license):
        self.about_authors.set_active(False)
        while Gtk.events_pending():
            Gtk.main_iteration()
        if self.about_license.get_active():
            self.about_notebook.set_current_page(2)
        else:
            self.about_notebook.set_current_page(0)

    def video_authors_toggle(self, video_authors):
        self.video_license.set_active(False)
        while Gtk.events_pending():
            Gtk.main_iteration()
        if self.video_authors.get_active():
            self.video_notebook.set_current_page(1)
        else:
            self.video_notebook.set_current_page(0)

    def video_license_toggle(self, video_license):
        self.video_authors.set_active(False)
        while Gtk.events_pending():
            Gtk.main_iteration()
        if self.video_license.get_active():
            self.video_notebook.set_current_page(2)
        else:
            self.video_notebook.set_current_page(0)

    def on_about_bashkaraoke(self, video_bashkaraoke):
        self.main_notebook.set_current_page(2)

    def onmouseover(self, show_the_video):
        self.show_the_video.set_label(_("Play the video again"))

    def onmouseout(self, show_the_video):
        self.show_the_video.set_label("")

    def on_video_show(self, show_the_video):
        self.show_the_video.set_label(_("You found me!"))
        while Gtk.events_pending():
            Gtk.main_iteration()
        time.sleep(0.5)
        self.fadeout()
        vvv = subprocess.Popen("python " + bkdir + "/intro.py", shell=True)
        vvv.wait()
        self.fadein()
    # Values
    def on_pitch(self, pitch_adj):
        pitch_v1 = str(int(self.pitch_adj.get_value()))
        if playing:
            self.playpause.set_icon_name("dialog-ok")
        else: pass

    def on_volume(self, volume_adj):
        volume_v1 = str(int(self.volume_adj.get_value()))
        if playing:
            self.playpause.set_icon_name("dialog-ok")
        else: pass
    def on_speed(self, speed_adj):
        speed_v1 = str(int(self.speed_adj.get_value()))
        if playing:
            self.playpause.set_icon_name("dialog-ok")
        else: pass

    # Settings window
    def on_settings(self, actionSettings):
        self.get_settings_both()
        self.check_modes()
        self.show_settings()
        self.main_notebook.set_current_page(1)

    def settings_ok(self, settings_ok):
        self.main_notebook.set_current_page(0)
        debug(_("Saving settings..."))
        self.save_settings()
        self.save_settings_both()
        debug(_("Settings saved."))
        self.get_settings_both()
        self.check_modes()
        self.show_settings()

    def settings_cancel(self, settings_cancel):
        self.main_notebook.set_current_page(0)
        debug(_("Reloading setting"))
        self.get_settings_both()
        self.check_modes()
        self.show_settings()
        debug(_("Setting reloaded"))

    def restore_all(self, restore_all):
        self.predefspeed.set_value(100)
        self.predefpitch.set_value(0)
        self.predefvolume.set_value(100)
        self.sung_color.set_active(self.int2color("red"))
        self.tosing_color.set_active(self.int2color("green"))
        self.font_button.set_font_name("Ubuntu 40")
        self.sf2_chooser.set_filename("/usr/local/share/timidity/PC51f.sf2")
        self.show_spectrum.set_active(False)
        self.extr_internal.set_active(True)
        self.extr_external.set_active(False)
        self.tolerance_combo.set_active(0)
        self.debug_switch.set_active(False)
        self.toggle_buc.set_active(False)
        self.toggle_gsw.set_active(False)
        self.toggle_gtk.set_active(True)
        self.winmode_combo.set_active(0)
        self.winmode_combo.set_sensitive(False)
        self.columns_adj.set_value(24)
        self.rows_adj.set_value(2)
        self.background_color.set_color(Gdk.color_parse("#000000"))
        self.lang_entry.set_text(str(os.environ.get("LC_CTYPE")))
        self.sep_entry.set_text("________________________________")
        self.show_lyrics.set_active(True)
        self.show_levels.set_active(True)
        self.show_channels.set_active(False)
        success(_("Settings restored"))

    def restore_speed(self, settings):
        self.predefspeed.set_value(100)
    def restore_pitch(self, settings):
        self.predefpitch.set_value(0)
    def restore_volume(self, settings):
        self.predefvolume.set_value(100)
    def restore_sung_color(self, settings):
        self.sung_color.set_color(Gdk.color_parse("#FF0000"))
    def restore_tosing_color(self, settings):
        self.tosing_color.set_color(Gdk.color_parse("#008000"))
    def restore_font_family(self, settings):
        self.font_button.set_font_name("Ubuntu 40")
    def restore_soundfont(self, settings):
        self.sf2_chooser.set_filename("/usr/local/share/timidity/PC51f.sf2")
    def restore_show_spectrum(self, settings):
        self.show_spectrum.set_active(False)
    def restore_extr(self, settings):
        self.extr_internal.set_active(True)
        self.extr_external.set_active(False)
    def restore_tolerance(self, settings):
        self.tolerance_combo.set_active(0)
    def restore_debug_mode(self, settings):
        self.debug_switch.set_active(False)
    def restore_winmode(self, settings):
        self.toggle_buc.set_active(False)
        self.toggle_gsw.set_active(False)
        self.toggle_gtk.set_active(True)
        self.winmode_combo.set_active(0)
        self.winmode_combo.set_sensitive(False)
    def restore_columns(self, settings):
        self.columns_adj.set_value(24)
    def restore_rows(self, settings):
        self.rows_adj.set_value(2)
    def restore_bg_color(self, settings):
        self.background_color.set_color(Gdk.color_parse("#000000"))
    def restore_lang(self, settings):
        self.lang_entry.set_text(str(os.environ.get("LC_CTYPE")))
    def restore_sep(self, settings):
        self.sep_entry.set_text("________________________________")
    def restore_show_lyrics(self, settings):
        self.show_lyrics.set_active(True)
    def restore_show_levels(self, settings):
        self.show_levels.set_active(True)
    def restore_show_channels(self, settings):
        self.show_channels.set_active(False)

    def get_settings_both(self):
        global show_debug
        f = open(user.home + '/.bashkaraoke/.bashkaraoke.rc', 'r')
        for line in f.readlines():
            if 'export rows=' in line:                                    # ROWS
                try:
                    self.ebg_rows = int(line.split('=', 1)[1])
                except:
                    warning(_("gtk_predefvolume value not found. Restoring default value..."))
                    self.ebg_rows = 24
                debug(_("Rows:") + " " + str(self.ebg_rows) + "  (" + str(len(str(self.ebg_rows))) + ")")
            elif 'export supermode=' in line:                        # SUPERMODE
                try:
                    self.ebg_winmode = str(line.split('=', 1)[1])[1:-2]
                except:
                    self.ebg_winmode = "auto"
                    warning(_("ebg_winmode value not found. Restoring default value..."))
                debug(_("Winmode:") + " " + self.ebg_winmode + "  (" + str(len(self.ebg_winmode)) + ")")
            elif 'export columns=' in line:                            # COLUMNS
                try:
                    self.ebg_columns = int(line.split('=', 1)[1])
                except:
                    warning(_("ebg_columns value not found. Restoring default value..."))
                    self.ebg_columns = 2                
                debug(_("Columns:") + " " + str(self.ebg_winmode) + "  (" + str(len(str(self.ebg_winmode))) + ")")
            elif 'export background=' in line:                      # BACKGROUND
                try:
                    self.ebg_bg_color = Gdk.color_parse(str(line.split('=', 1)[1])[1:-2])
                except:
                    warning(_("ebg_bg_color value not found. Restoring default value..."))
                    self.ebg_bg_color = Gdk.color_parse("#000000")
                debug(_("Background:") + " " + self.ebg_bg_color.to_string() + "  (" + str(len(self.ebg_bg_color.to_string())) + ")")
            elif 'export foreground_1=' in line:                       # TO SING
                try:
                    self.ebg_tosing_color = str(line.split('=', 1)[1])[1:-2]
                except:
                    warning(_("ebg_tosing_color value not found. Restoring default value..."))
                    self.ebg_tosing_color = "green"
                debug(_("To sing text color:") + " " + str(line.split('=', 1)[1])[1:-2] + "  (" + str(len(self.ebg_tosing_color)) + ")")
            elif 'export foreground_2=' in line:                          # SUNG
                try:
                    self.ebg_sung_color = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_sung_color value not found. Restoring default value..."))
                    self.ebg_sung_color = "red"
                debug(_("Sung text color:") + " " + str(line.split('=', 1)[1])[1:-2] + "  (" + str(len(self.ebg_sung_color)) + ")")
            elif 'export soundfont=' in line:                        # SOUNDFONT
                try:
                    self.ebg_soundfont = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_soundfont value not found. Restoring default value..."))
                    self.ebg_soundfont = "/usr/local/share/timidity/PC51f.sf2"
                debug(_("SoundFont:") + " " + self.ebg_soundfont + "  (" + str(len(self.ebg_winmode)) + ")")
            elif 'export extract_way=' in line:                    # EXTRACT WAY
                try:
                    self.ebg_extract_way = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_extract_way value not found. Restoring default value..."))
                    self.ebg_extract_way = "internal"
                debug(_("Extract way:") + " " + self.ebg_extract_way + "  (" + str(len(self.ebg_extract_way)) + ")")
            elif 'export char_encoding=' in line:               # CHAR. ENCODING
                try:
                    self.ebg_char_enc = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_char_enc value not found. Restoring default value..."))
                    self.ebg_char_enc = "ISO-8859-1"
                debug(_("Character encoding:") + " " + self.ebg_char_enc + "  (" + str(len(self.ebg_char_enc)) + ")")
            elif 'export use_buc=' in line:                            # USE BUC
                try:
                    usebuc = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_use_buc value not found. Restoring default value..."))
                    usebuc = ""
                if usebuc == "true":
                    self.ebg_use_buc = True
                elif usebuc == "false":
                    self.ebg_use_buc = False
                else:
                    self.ebg_use_buc = False
                debug(_("Use BUC:") + " " + str(self.ebg_use_buc))
            elif 'export use_gtk=' in line:                            # USE GTK
                try:
                    usegtk = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_use_gtk value not found. Restoring default value..."))
                    usegtk = ""
                if usegtk == "true":
                    self.ebg_use_gtk = True
                elif usegtk == "false":
                    self.ebg_use_gtk = False
                else:
                    self.ebg_use_gtk = True
                debug(_("Use GTK:") + " " + str(self.ebg_use_gtk))
            elif 'export use_gsw=' in line:  # USE GTKDIALOG SINGLE WINDOW (GSW)
                try:
                    usegsw = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_use_gsw value not found. Restoring default value..."))
                    usegsw = ""
                if usegsw == "true":
                    self.ebg_use_gsw = True
                elif usegsw == "false":
                    self.ebg_use_gsw = False
                else:
                    self.ebg_use_gws = False
                debug(_("use GSW (Gtkdialog Single Window):") + " " + str(self.ebg_use_gsw))
            elif 'export LANG=' in line:                              # LANGUAGE
                try:
                    self.ebg_lang = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_lang value not found. Restoring default value..."))
                    self.ebg_lang = str(os.environ.get("LC_CTYPE"))
                if self.ebg_lang == "" or self.ebg_lang == None:
                    self.ebg_lang = "en_US.UTF-8"
                debug(_("Language:") + " " + self.ebg_lang)
            elif 'export karaoke_window=' in line:              # KARAOKE WINDOW
                try:
                    k_win = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_karaoke_win value not found. Restoring default value..."))
                    k_win = ""
                if k_win == "true":
                    self.ebg_karaoke_win = True
                elif k_win == "false":
                    self.ebg_karaoke_win = False
                else:
                    self.ebg_karaoke_win = True
                debug(_("Show karaoke window:") + " " + str(self.ebg_karaoke_win))
            elif 'export levels_window=' in line:                # LEVELS WINDOW
                try:
                    l_win = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_levels_win value not found. Restoring default value..."))
                    l_win = ""
                if l_win == "true":
                    self.ebg_levels_win = True
                elif l_win == "false":
                    self.ebg_levels_win = False
                else:
                    self.ebg_levels = False
                debug(_("Show levels window:") + " " + str(self.ebg_levels_win))
            elif 'export channels_window=' in line:            # CHANNELS WINDOW
                try:
                    c_win = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_channels_win value not found. Restoring default value..."))
                    c_win = ""
                if c_win == "true":
                    self.ebg_channels_win = True
                elif c_win == "false":
                    self.ebg_channels_win = False
                else:
                    self.ebg_channels = False
                debug(_("Show channels window:") + " " + str(self.ebg_channels_win))
            elif 'export spectrogram_window=' in line:         # SPECTRUM WINDOW
                try:
                    s_win = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_spectrum_win value not found. Restoring default value..."))
                    s_win = ""
                if s_win == "true":
                    self.ebg_spectrum_win = True
                elif s_win == "false":
                    self.ebg_spectrum_win = False
                else:
                    self.ebg_spectrum_win = False
                debug(_("Show spectrogram window:") + " " + str(self.ebg_spectrum_win))
            elif 'export error_correction_chars=' in line:           # TOLERANCE
                try:
                    self.ebg_tolerance = int(line.split("=", 1)[1])
                except:
                    try:
                        self.ebg_tolerance = int(line.split("=", 1)[1][1:-2])
                    except:
                        warning(_("ebg_tolerance value not found. Restoring default value..."))
                        self.ebg_tolerance = 0
                debug(_("Tolerance level:") + " " + str(self.ebg_tolerance) + "  (" + str(len(str(self.ebg_tolerance))) + ")")
            elif 'export delimiter=' in line:                        # DELIMITER
                try:
                    self.ebg_delimiter = str(line.split("=", 1)[1][1:-2])
                except:
                    warning(_("ebg_delimiter value not found. Restoring default value..."))
                    self.ebg_delimiter = "______________________________________"
                debug(_("EBG menu delimiter:") + " " + self.ebg_delimiter + "  (" + str(len(self.ebg_delimiter)) + ")")
            elif 'export font=' in line:                                  # FONT
                try:
                    self.ebg_font = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("ebg_font value not found. Restoring default value..."))
                    self.ebg_font = "-b&h-lucidatypewriter-medium-r-normal-sans-70-*-*-*-*-*-iso8859-1"
                debug(_("Font Family (EBG mode):") + " " + self.ebg_font + "  (" + str(len(self.ebg_font)) + ")")
            elif 'export predefvolume=' in line:                        # VOLUME
                try:
                    self.gtk_predefvolume = int(line.split('=', 1)[1])
                except:
                    warning(_("gtk_predefvolume value not found. Restoring default value..."))
                    self.gtk_predefvolume = 100
                debug(_("Default volume:") + " " + str(self.gtk_predefvolume) + "  (" + str(len(str(self.gtk_predefvolume))) + ")")
            elif 'export predefpitch=' in line:                          # PITCH
                try:
                    self.gtk_predefpitch = int(line.split('=', 1)[1])
                except:
                    warning(_("gtk_predefpitch value not found. Restoring default value..."))
                    self.gtk_predefpitch = 0
                debug(_("Default pitch:") + " " + str(self.gtk_predefpitch) + "  (" + str(len(str(self.gtk_predefpitch))) + ")")
            elif 'export predefspeed=' in line:                          # SPEED
                try:
                    self.gtk_predefspeed = int(line.split('=', 1)[1])
                except:
                    warning(_("gtk_predefspeed value not found. Restoring default value..."))
                    self.gtk_predefspeed = 100
                debug(_("Default speed:") + " " + str(self.gtk_predefspeed) + "  (" + str(len(str(self.gtk_predefspeed))) + ")")
            elif 'export font_family=' in line:                    # FONT FAMILY
                try:
                    self.gtk_font_family = str(line.split("=", 1)[1])[1:-2]
                except:
                    warning(_("gtk_font_family value not found. Restoring default value..."))
                    self.gtk_font_family = "Ubuntu 50"
                debug(_("Font Family (GTK mode):") + " " + self.gtk_font_family + "  (" + str(len(self.gtk_font_family)) + ")")
            elif 'export show_debug=' in line:                           # DEBUG
                try:
                    self.show_debug = int(line.split("=", 1)[1])
                except:
                    try:
                        self.show_debug = int(line.split("=", 1)[1][1:-2])
                    except:
                        warning(_("show_debug value not found. Restoring default value..."))
                        self.show_debug = 0
                show_debug = bool(self.show_debug)
                debug(("Debug mode:") + " " + str(show_debug))
        f.close()
        try:
            self.exists(self.gtk_predefspeed)
        except:
            self.gtk_predefspeed = 100
            warning(_("gtk_predefvolume value not found. Restoring default value..."))
        try:
            self.exists(self.gtk_predefpitch)
        except:
            self.gtk_predefpitch = 0
            warning(_("gtk_predefpitch value not found. Restoring default value..."))
        try:
            self.exists(self.gtk_predefvolume)
        except:
            self.gtk_predefvolume = 100
            warning(_("gtk_predefspeed value not found. Restoring default value..."))
        try:
            self.exists(self.ebg_sung_color)
        except:
            self.ebg_sung_color = "red"
            warning(_("ebg_sung_color color not found. Restoring default value..."))
        try:
            self.exists(self.ebg_tosing_color)
        except:
            self.ebg_tosing_color = "green"
            warning(_("ebg_tosing_color not found. Restoring default value..."))
        try:
            self.exists(self.gtk_font_family)
        except:
            self.gtk_font_family = "Ubuntu 50"
            warning(_("gtk_font_family not found. Restoring default value..."))
        try:
            self.exists(self.ebg_soundfont)
        except:
            self.ebg_soundfont = "/usr/local/share/timidity/PC51f.sf2"
            warning(_("ebg_soundfont not found. Restoring default value..."))
        try:
            self.exists(self.ebg_spectrum_win)
        except:
            self.ebg_spectrum_win = False
            warning(_("ebg_spectrum_win not found. Restoring default value..."))
        try:
            self.exists(self.ebg_extract_way)
        except:
            self.ebg_extract_way = "internal"
            warning(_("ebg_extract_way not found. Restoring default value..."))
        try:
            self.exists(self.ebg_tolerance)
        except:
            self.ebg_tolerance = 0
            warning(_("ebg_tolerance not found. Restoring default value..."))
        try:
            self.exists(self.show_debug)
            self.exists(show_debug)
        except:
            self.show_debug = 0
            show_debug = False
            warning(_("show_debug not found. Restoring default value..."))
        try:
            self.exists(self.ebg_use_buc)
        except:
            self.ebg_use_buc = False
            warning(_("ebg_use_buc not found. Restoring default value..."))
        try:
            self.exists(self.ebg_use_gtk)
        except:
            self.ebg_use_gtk = True
            warning(_("ebg_use_gtk not found. Restoring default value..."))
        try:
            self.exists(self.ebg_use_gsw)
        except:
            self.ebg_use_gsw = False
            warning(_("ebg_use_gsw not found. Restoring default value..."))
        try:
            self.exists(self.ebg_winmode)
        except:
            self.ebg_winmode = "auto"
            warning(_("ebg_winmode not found. Restoring default value..."))
        try:
            self.exists(self.ebg_columns)
        except:
            self.ebg_columns = 24
            warning(_("ebg_columns not found. Restoring default value..."))
        try:
            self.exists(self.ebg_rows)
        except:
            self.ebg_rows = 2
            warning(_("ebg_rows not found. Restoring default value..."))
        try:
            self.exists(self.ebg_bg_color)
        except:
            self.ebg_bg_color = Gdk.color_parse("#000000")
            warning(_("ebg_bg_color not found. Restoring default value..."))
        try:
            self.exists(self.ebg_lang)
        except:
            self.ebg_lang = os.get.environ("LANG")
            warning(_("ebg_lang not found. Restoring default value..."))
        try:
            self.exists(self.ebg_delimiter)
        except:
            self.ebg_delimiter = "______________________________________________"
            warning(_("ebg_delimiter not found. Restoring default value..."))
        try:
            self.exists(self.ebg_karaoke_win)
        except:
            self.ebg_karaoke_win = True
            warning(_("ebg_karaoke_win not found. Restoring default value..."))
        try:
            self.exists(self.ebg_levels_win)
        except:
            self.ebg_levels_win = True
            warning(_("ebg_levels_win not found. Restoring default value..."))
        try:
            self.exists(self.ebg_channels_win)
        except:
            self.ebg_channels_win = True
            warning(_("ebg_channels_win not found. Restoring default value..."))
        try:
            self.exists(self.ebg_font)
        except:
            self.ebg_font = "-b&h-lucidatypewriter-medium-r-normal-sans-70-*-*-*-*-*-iso8859-1"
            warning(_("ebg_font not found. Restoring default value..."))

    def check_modes(self):
        devnull = open("/dev/null", "a")
        self.winmode_combo.remove_all()
        gtkdialog = subprocess.Popen(["which", "gtkdialog"], stdout=devnull)
        buc = subprocess.Popen(["which", "buc"], stdout=devnull)
        zenity = subprocess.Popen(["which", "zenity"], stdout=devnull)
        xdialog = subprocess.Popen(["which", "xdialog"], stdout=devnull)
        kdialog = subprocess.Popen(["which", "kdialog"], stdout=devnull)
        dialog = subprocess.Popen(["which", "dialog"], stdout=devnull)
        self.winmode_combo.append_text("Auto")
        self.toggle_gtk.set_tooltip_markup(_("Toggle Bash!Karaoke GTK mode (current)"))
        gtkdialog.wait()
        try:
            if int(gtkdialog.poll()) > 0:
                self.toggle_gsw.set_sensitive(False)
                self.toggle_gsw.set_tooltip_markup(_("You must install \"gtkdialog\" in order\nto use Gtkdialog Single Window"))
                self.winmode_combo.append_text(_("Not available (gtkdialog)"))
                self.gtkdialog_is_avail = False
            elif int(gtkdialog.poll()) == 0:
                self.toggle_gsw.set_sensitive(True)
                self.toggle_gsw.set_tooltip_markup(_("Toggle Gtkdialog Single Window mode"))
                self.winmode_combo.append_text("Gtkdialog")
                self.gtkdialog_is_avail = True
        except: pass
        buc.wait
        try:
            if int(buc.poll()) > 0:
                self.toggle_buc.set_sensitive(False)
                self.toggle_buc.set_tooltip_markup(_("You must install \"buc\"\nin order to use BUC mode"))
                self.buc_is_avail = False
            elif int(buc.poll()) == 0:
                self.toggle_buc.set_sensitive(True)
                self.toggle_buc.set_tooltip_markup(_("Toggle Buc mode"))
                self.buc_is_avail = True
        except: pass
        zenity.wait()
        try:
            if int(zenity.poll()) == 0:
                self.winmode_combo.append_text("Zenity")
                self.zenity_is_avail = True
            elif int(zenity.poll()) > 0:
                self.winmode_combo.append_text(_("Not available (zenity)"))
                self.zenity_is_avail = False
        except: pass
        xdialog.wait()
        try:
            if int(xdialog.poll()) == 0:
                self.winmode_combo.append_text("XDialog")
                self.xdialog_is_avail = True
            elif int(xdialog.poll()) > 0:
                self.winmode_combo.append_text(_("Not available (xdialog)"))
                self.xdialog_is_avail = False
        except: pass
        kdialog.wait()
        try:
            if int(kdialog.poll()) == 0:
                self.winmode_combo.append_text("KDialog")
                self.kdialog_is_avail = True
            elif int(kdialog.poll()) > 0:
                self.winmode_combo.append_text(_("Not available (kdialog)"))
                self.kdialog_is_avail = False
        except: pass
        dialog.wait()
        try:
            if int(dialog.poll()) == 0:
                self.winmode_combo.append_text("Dialog")
                self.dialog_is_avail = True
            elif int(dialog.poll()) > 0:
                self.winmode_combo.append_text(_("Not available (dialog)"))
                self.dialog_is_avail = False
            self.winmode_combo.set_tooltip_markup(_("Choose an EBG dialog mode"))
        except: pass
        devnull.close()
        debug("\r                                ")
        debug(_("Buc is installed:") + " " + str(self.buc_is_avail))
        debug(_("Gtkdialog is installed:") + " " + str(self.gtkdialog_is_avail))
        debug(_("Zenity is installed:") + " " + str(self.zenity_is_avail))
        debug(_("XDialog is installed:") + " " + str(self.xdialog_is_avail))
        debug(_("KDialog is installed:")+ " " + str(self.kdialog_is_avail))
        debug(_("Dialog is installed:") + " " + str(self.dialog_is_avail))


    def show_settings(self):
        self.predefspeed.set_value(int(self.gtk_predefspeed))
        self.predefpitch.set_value(int(self.gtk_predefpitch))
        self.predefvolume.set_value(int(self.gtk_predefvolume))
        self.speed_adj.set_value(int(self.gtk_predefspeed))
        self.pitch_adj.set_value(int(self.gtk_predefpitch))
        self.volume_adj.set_value(int(self.gtk_predefvolume))
        self.sung_color.set_active(self.color2int(self.ebg_sung_color))
        self.tosing_color.set_active(self.color2int(self.ebg_tosing_color))
        self.font_button.set_font_name(self.gtk_font_family)
        self.sf2_chooser.set_filename(self.ebg_soundfont)
        self.show_spectrum.set_active(self.ebg_spectrum_win)
        if self.ebg_extract_way == "internal":
            self.extr_internal.set_active(True)
            self.extr_external.set_active(False)
        elif self.ebg_extract_way == "external":
            self.extr_internal.set_active(False)
            self.extr_external.set_active(True)
        else:
            self.extr_internal.set_active(True)
            self.extr_external.set_active(False)
        self.tolerance_combo.set_active(int(self.ebg_tolerance))
        self.debug_switch.set_active(show_debug)
        if self.buc_is_avail:
            self.toggle_buc.set_active(self.ebg_use_buc)
        else:
            self.toggle_buc.set_active(False)
        if self.gtkdialog_is_avail:
            self.toggle_gsw.set_active(self.ebg_use_gsw)
        else:
            self.toggle_gsw.set_active(False)
        self.toggle_gtk.set_active(self.ebg_use_gtk)
        if self.ebg_winmode == "auto":
            self.winmode_combo.set_active(0)
        elif self.ebg_winmode == "gtkdialog" and self.gtkdialog_is_avail:
            self.winmode_combo.set_active(1)
        elif self.ebg_winmode == "zenity" and self.zenity_is_avail:
            self.winmode_combo.set_active(2)
        elif self.ebg_winmode == "xdialog" and self.xdialog_is_avail:
            self.winmode_combo.set_active(3)
        elif self.ebg_winmode == "kdialog" and self.kdialog_is_avail:
            self.winmode_combo.set_active(4)
        elif self.ebg_winmode == "dialog" and self.dialog_is_avail:
            self.winmode_combo.set_active(5)
        else:
            self.winmode_combo.set_active(0)
        if self.toggle_gtk.get_active():
            self.winmode_combo.set_sensitive(False)
        elif not self.toggle_gtk.get_active():
            self.winmode_combo.set_sensitive(True)
        self.columns_adj.set_value(self.ebg_columns)
        self.rows_adj.set_value(self.ebg_rows)
        self.background_color.set_color(self.ebg_bg_color)
        self.lang_entry.set_text(self.ebg_lang)
        self.sep_entry.set_text(self.ebg_delimiter)
        self.show_lyrics.set_active(self.ebg_karaoke_win)
        self.show_levels.set_active(self.ebg_levels_win)
        self.show_channels.set_active(self.ebg_channels_win)

    def internal_toggle(self, extr_internal):
        if extr_internal.get_active() == True:
            self.extr_external.set_active(False)
        elif extr_internal.get_active() == False:
            self.extr_external.set_active(True)

    def external_toggle(self, extr_external):
        if extr_external.get_active() == True:
            self.extr_internal.set_active(False)
        elif extr_external.get_active() == False:
            self.extr_internal.set_active(True)

    def toggle_use_buc(self, toggle_buc):
        if self.toggle_buc.get_active():
            self.toggle_gtk.set_active(False)
            self.toggle_gsw.set_active(False)
        if self.toggle_gtk.get_active():
            self.winmode_combo.set_sensitive(False)
        elif not self.toggle_gtk.get_active():
            self.winmode_combo.set_sensitive(True)

    def toggle_use_gtk(self, toggle_gtk):
        if self.toggle_gtk.get_active():
            self.toggle_buc.set_active(False)
            self.toggle_gsw.set_active(False)
        if self.toggle_gtk.get_active():
            self.winmode_combo.set_sensitive(False)
        elif not self.toggle_gtk.get_active():
            self.winmode_combo.set_sensitive(True)

    def toggle_use_gsw(self, toggle_gsw):
        if self.toggle_gsw.get_active():
            self.toggle_gtk.set_active(False)
            self.toggle_buc.set_active(False)
        if self.toggle_gtk.get_active():
            self.winmode_combo.set_sensitive(False)
        elif not self.toggle_gtk.get_active():
            self.winmode_combo.set_sensitive(True)


    def save_settings(self):
        self.gtk_predefspeed = int(self.predefspeed.get_value())
        self.gtk_predefpitch = int(self.predefpitch.get_value())
        self.gtk_predefvolume = int(self.predefvolume.get_value())
        self.ebg_sung_color = self.int2color(self.sung_color.get_active())
        self.ebg_tosing_color = self.int2color(self.tosing_color.get_active())
        self.gtk_font_family = self.font_button.get_font_name()
        while re.search(".\s\d+", self.gtk_font_family) is not None or re.search(".\s+$", self.gtk_font_family) is not None:
            self.gtk_font_family = self.gtk_font_family[:-1]
        self.ebg_soundfont = self.sf2_chooser.get_filename()
        self.ebg_spectrum_win = self.show_spectrum.get_active()
        if self.extr_internal.get_active() == True and self.extr_external.get_active() == False:
            self.ebg_extract_way = "internal"
        elif self.extr_external.get_active() == True and self.extr_internal.get_active() == False:
            self.ebg_extract_way = "external"
        else:
            self.ebg_extract_way = "internal"
        self.ebg_tolerance = self.tolerance_combo.get_active()
        if self.debug_switch.get_active() == False:
            self.show_debug = 0
        elif self.debug_switch.get_active() == True:
            self.show_debug = 1
        show_debug = self.debug_switch.get_active()
        self.ebg_use_buc = self.bool2string(self.toggle_buc.get_active())
        self.ebg_use_gtk = self.bool2string(self.toggle_gtk.get_active())
        self.ebg_use_gsw = self.bool2string(self.toggle_gsw.get_active())
        if self.winmode_combo.get_active() == 0:
            self.ebg_winmode = "auto"
        elif self.winmode_combo.get_active() == 1 and self.gtkdialog_is_avail:
            self.ebg_winmode = "gtkdialog"
        elif self.winmode_combo.get_active() == 2 and self.zenity_is_avail:
            self.ebg_winmode = "zenity"
        elif self.winmode_combo.get_active() == 3 and self.xdialog_is_avail:
            self.ebg_winmode = "xdialog"
        elif self.winmode_combo.get_active() == 4 and self.kdialog_is_avail:
            self.ebg_winmode = "kdialog"
        elif self.winmode_combo.get_active() == 5 and self.dialog_is_avail:
            self.ebg_winmode = "dialog"
        else:
            self.ebg_winmode = "auto"
        self.ebg_columns = int(self.columns_adj.get_value())
        self.ebg_rows = int(self.rows_adj.get_value())
        self.ebg_bg_color = self.background_color.get_color()
        self.ebg_lang = self.lang_entry.get_text()
        self.ebg_delimiter = self.sep_entry.get_text()
        self.ebg_karaoke_win = self.show_lyrics.get_active()
        self.ebg_levels_win = self.show_levels.get_active()
        self.ebg_channels_win = self.show_channels.get_active()

    def save_settings_both(self):
        ebggg = open(user.home + "/.bashkaraoke/.bashkaraoke.rc", "w")
        ebggg.write('export use_gsw="' + str(self.ebg_use_gsw) + '"\nexport use_gtk="' + str(self.ebg_use_gtk) + '"\nexport use_buc="' + str(self.ebg_use_buc) + '"\nexport supermode="' + str(self.ebg_winmode) + '"\nexport karaoke_window="' + str(self.bool2string(bool(self.ebg_karaoke_win))) + '"\nexport levels_window="' + str(self.bool2string(bool(self.ebg_levels_win))) + '"\nexport channels_window="' + str(self.bool2string(bool(self.ebg_channels_win))) + '"\nexport spectrogram_window="' + str(self.bool2string(bool(self.ebg_spectrum_win))) + '"\nexport rows=' + str(self.ebg_rows) + '\nexport columns=' + str(self.ebg_columns) + '\nexport font="' + str(self.ebg_font) + '"\nexport background="' + str(self.ebg_bg_color.to_string()) + '"\nexport foreground_1="' + str(self.ebg_tosing_color) + '"\nexport foreground_2="' + str(self.ebg_sung_color) + '"\nexport soundfont="' + str(self.ebg_soundfont) + '"\nexport extract_way="' + str(self.ebg_extract_way) + '"\nexport error_correction_chars=' + str(self.ebg_tolerance) + '\nexport LANG="' + str(self.ebg_lang) + '"\nexport char_encoding="' + str(self.ebg_char_enc) + '"\nexport delimiter="' + str(self.ebg_delimiter) + '"\nexport predefvolume=' + str(self.gtk_predefvolume) + '\nexport predefpitch=' + str(self.gtk_predefpitch) + '\nexport predefspeed=' + str(self.gtk_predefspeed) + '\nexport font_family="' + str(self.gtk_font_family) + '"\nexport show_debug=' + str(self.show_debug) + "\n")
        ebggg.close()

    def exists(self, var):
        pass

    def bool2string(self, txt):
        if txt == True:
            return str("true")
        elif txt == False:
            return str("false")

    def color2int(self, color):
        if color == "black":
            return 0
        elif color == "red":
            return 1
        elif color == "green":
            return 2
        elif color == "yellow":
            return 3
        elif color == "blue":
            return 4
        elif color == "magenta":
            return 5
        elif color == "cyan":
            return 6
        elif color == "white":
            return 7
        else: raise TypeError(_("The color you gave is not valid:") + " " + str(color))

    def int2color(self, color):
        if color == 0:
            return "black"
        elif color == 1:
            return "red"
        elif color == 2:
            return "green"
        elif color == 3:
            return "yellow"
        elif color == 4:
            return "blue"
        elif color == 5:
            return "magenta"
        elif color == 6:
            return "cyan"
        elif color == 7:
            return "white"
        else: raise TypeError(_("The number you gave is not valid:") + " " + str(color))

    def check_oxygen(self):
        try:
            import apt
            global apt
            debian = True
        except ImportError:
            debian = False
        if debian:
            self.check_oxygen_apt()
        elif not debian:
            self.check_oxygen_notapt()
        else:
            self.check_oxygen_notapt()

    def check_oxygen_apt(self):
        cache = apt.Cache()
        try:
            engines = cache["gtk3-engines-oxygen"]
            installed = engines.is_installed
        except:
            installed = False
        if not installed:
            question = subprocess.Popen(["bash", "/usr/share/bashkaraoke-gtk/oxyquestion.bash"])
            question.wait()
            if int(question.poll()) == 0:
                kdesudo = subprocess.Popen(["which", "kdesudo"])
                kdesudo.wait()
                if int(kdesudo.poll()) == 0:
                    run = subprocess.Popen(["kdesudo", "--comment", _("You need to authenticate in order to install the packages."), "-i", "bashkaraoke-gtk", "--", "bash", "/usr/share/bashkaraoke-gtk/gtk3_oxygen.bash"])
                    run.wait()
                else:
                    run = subprocess.Popen(["gksudo", "-k", "-D", "Bash!Karaoke GTK", "bash", "/usr/share/bashkaraoke-gtk/gtk3_oxygen.bash"])
                    run.wait()
                if not os.path.exists(user.home + "/.config/gtk-3.0"):
                    os.mkdir(user.home + "/.config/gtk-3.0")
                else: pass
                ini = open(user.home + "/.config/gtk-3.0/settings.ini", "w")
                ini.write("[Settings]\ngtk-theme-name = oxygen-gtk\n# " + _("This file has been written by Bash!Karaoke GTK to allow using it with KDE Oxygen") + "\n")
                ini.close()
                if int(run.poll()) == 0:
                    self.restart()
                else:
                    uff = subprocess.Popen(["bash", "/usr/share/bashkaraoke-gtk/uff.bash"])
                    uff.wait()
            else:
                ohno = subprocess.Popen(["bash", "/usr/share/bashkaraoke-gtk/ohno.bash"])
                ohno.wait()
    def check_oxygen_notapt(self):
        if not os.path.exists("/usr/lib/gtk-3.0/3.0.0/theming-engines/liboxygen-gtk.so"):
            a = os.path.exists(user.home + "/.bashkaraoke/.dontshow")
        else: a = True
        if not a:          
            oxy = subprocess.Popen(["bash", "/usr/share/bashkaraoke-gtk/oxynodebian.bash"])
            oxy.wait()
            if oxy.poll() == 1:
                f = open(user.home + "/.bashkaraoke/.dontshow", "w")
                f.write(_("It seems that you are using KDE on a APT-less distro.\nYou need an optional packageto run Bash!Karaoke.\nIn Ubuntu it is in a PPA and its name is «gtk3-engines-oxygen».\n\nThe following are the files provided by the package above:\n\n/.\n/usr\n/usr/bin\n/usr/bin/oxygen-gtk3-demo\n/usr/lib\n/usr/lib/gtk-3.0\n/usr/lib/gtk-3.0/3.0.0\n/usr/lib/gtk-3.0/3.0.0/theming-engines\n/usr/lib/gtk-3.0/3.0.0/theming-engines/liboxygen-gtk.so\n/usr/share\n/usr/share/doc\n/usr/share/doc/gtk3-engines-oxygen\n/usr/share/doc/gtk3-engines-oxygen/README.gz\n/usr/share/doc/gtk3-engines-oxygen/TODO\n/usr/share/doc/gtk3-engines-oxygen/changelog.Debian.gz\n/usr/share/doc/gtk3-engines-oxygen/copyright\n/usr/share/themes\n/usr/share/themes/oxygen-gtk\n/usr/share/themes/oxygen-gtk/gtk-3.0\n/usr/share/themes/oxygen-gtk/gtk-3.0/argb-apps.conf\n/usr/share/themes/oxygen-gtk/gtk-3.0/gtk.css\n/usr/share/themes/oxygen-gtk/gtk-3.0/icons4\n/usr/share/themes/oxygen-gtk/gtk-3.0/kdeglobals\n/usr/share/themes/oxygen-gtk/gtk-3.0/oxygenrc\n/usr/share/themes/oxygen-gtk/gtk-3.0/special-icons\n/usr/share/themes/oxygen-gtk/gtk-3.0/special-icons/standardbutton-closetab-16.png\n/usr/share/themes/oxygen-gtk/gtk-3.0/special-icons/standardbutton-closetab-down-16.png\n/usr/share/themes/oxygen-gtk/gtk-3.0/special-icons/standardbutton-closetab-hover-16.png"))
                f.close()
            else: pass
        else: pass


    # Play / Stop
    def playpause(self, playpause):
        global playing, e_dagli
        l_open = _("You didn't open a song; open a song and retry")
        l_open2 = _("Open a song")
        if filename == "" and not self.playpause.get_icon_name() == "dialog-ok":
            if e_dagli == 0:
                self.openlabel.set_label('''<span font="40" weight="light" color="red">''' + l_open2 + "!" + '''</span>''')
                warning(l_open)
                e_dagli = 1
            elif e_dagli == 1:
                self.openlabel.set_label('''<span font="40" color="red">''' + l_open2 + "!!" + '''</span>''')
                warning(l_open)
                e_dagli = 2
            elif e_dagli == 2:
                self.openlabel.set_label('''<span font="40" weight="bold" color="red">''' + l_open2 + '''!!!</span>''')
                warning(l_open)
            else:
                self.openlabel.set_label('''<span font="40" weight="light" color="red">''' + l_open2 + '''!</span>''')
                warning(l_open)
        else:
            if self.playpause.get_icon_name() == "dialog-ok":
                if playing:
                    playing = False
                    self.stop()
                    self.play()
                    playing = True
                    self.playpause.set_icon_name("media-playback-stop")
                    self.openbar.hide()
                    debug(_("Song restarted with new parameters"))
                elif not playing:
                    self.playpause.set_icon_name("media-playback-start")
            elif self.playpause.get_icon_name() == "media-playback-start":
                self.play()
                self.openbar.hide()
                self.playpause.set_icon_name("media-playback-stop")
                playing = True
            elif self.playpause.get_icon_name() == "media-playback-stop":
                self.stop()
                self.openbar.show()
                self.playpause.set_icon_name("media-playback-start")
                playing = False
                self.lyrics.set_label('''<span font="60" weight="light">Bash!Karaoke</span>\n<span font="20" weight="light">''' + self.l_btm + '''</span>''')
#            self.playpause.get_icon_name()

    # Open song
    def on_open(self, actionApri):
        self.openfile()
    def on_open2(self, openbutton):
        self.openfile()
    def openfile(self):
        dialog = Gtk.FileChooserDialog(_("Choose a song..."), self.actionApri.get_toplevel(), Gtk.FileChooserAction.OPEN)
        dialog.add_button(Gtk.STOCK_CANCEL, 0)
        dialog.add_button(Gtk.STOCK_OPEN, 1)
        dialog.set_default_response(1)

        dialog.set_filter(self.filefilter)        
        if dialog.run() == 1:
            global filename
            filename = str('"' + dialog.get_filename() + '"')
        self.openlabel.set_label('''<span font="40" weight="light">''' + _("Open a song") + '''</span>''')
        dialog.destroy()

    # Play
    def play(self):
        # Read values
        pitch_v = str(int(self.pitch_adj.get_value()))
        volume_v = str(int(self.volume_adj.get_value()))
        speed_v = str(int(self.speed_adj.get_value()))
        debug("Play")
        global playing
        thread.start_new_thread(self.bk, (filename, user.home + "/.bashkaraoke/.lyrics", self.gtk_font_family, volume_v, pitch_v, speed_v))
        self.refresh_label()
        GObject.timeout_add(50, self.refresh_label)
        time.sleep(0.5)
        self.lyrics.set_label('''<span font="60" weight="light">Bash!Karaoke</span>\n<span font="20" weight="light">''' + self.l_btm + '''</span>''')

    # Refresh lyrics
    def refresh_label(self):
        f = open(user.home + "/.bashkaraoke/.lyrics", "r")
        self.lyrics.set_label(f.read())
        f.close()
        global playing
        if playing:
            return True
        elif not playing:
            return False
            self.lyrics.set_label('''<span font="60" weight="light">Bash!Karaoke</span>\n<span font="20" weight="light">''' + self.l_btm + '''</span>''')
            f1 = open(user.home + "/.lyrics", "w")
            f1.write("""<span font="50" weight="light">""" + l_loading + """</span>""")
            f1.close()

    # Run bashkaraoke_OUT
    def bk(self, fname, lyrfile, font, vol, pit, spe):
        global playing
        args = '''bash /usr/local/share/bashkaraoke/bashkaraoke_FILES/bashkaraoke_OUT -o "s" -f "''' + lyrfile + '''" -F "''' + font + "50" + '''" -v ''' + vol + " -p " + pit + " -t " + spe + ''' ''' + fname
        debug(_("bashkaraoke_OUT arguments:") + " " + str(args))
        self.bk_out = subprocess.Popen(args + " >> /dev/null", shell=True)
        debug(_("Back-end (bashkaraoke_OUT) started"))
        self.processid = int(self.bk_out.pid)
        debug(str(self.processid))
        self.bk_out.wait()
        self.playpause.set_icon_name("media-playback-start")
        self.stop()
        playing = False
        debug(_("bashkaraoke_OUT exited"))
        self.lyrics.set_label('''<span font="60">Bash!Karaoke</span>\n<span font="20" weight="light">''' + self.l_btm + '''</span>''')
        f2 = open(user.home + "/.bashkaraoke/.lyrics", "w")
        f2.write("""<span font="50" weight="light">""" + l_loading + """</span>""")
        f2.close()

    # Stop song
    def stop(self):
        global playing
        playing = False
#        killer = subprocess.Popen("kill " + str(self.processid), shell=True)
#        killer.wait()
        pidd = open(user.home + "/.bashkaraoke/.pid")
        piddd = str(pidd.read())
        pidd.close()
        l_killed = _("bashkaraoke_OUT killed succesfully. This was its PID:")
        l_not_killed = _("bashkaraoke_OUT not killed")
        try:
            pid = int(piddd)
            number = True
        except:
            number = False
        if number:
            try: os.kill(pid, "SIGKILL")
            except:
                killer = subprocess.Popen(["kill", str(pid)])
                killer.wait()
                if str(self.bk_out.poll()) == "":
                    try: os.kill(self.processid, "SIGKILL")
                    except:
                        killer = subprocess.Popen(["kill", str(self.processid)])
                        killer.wait()
        if str(self.bk_out.poll()) == "":
            self.bk_out.terminate()
        else:
            killed = True
            debug(l_killed + " " + str(self.processid))
        if self.bk_out.poll() == "":
            warning(l_not_killed)
            self.bk_out.kill()
        else:
            if not killed:
                success(l_killed + " " + str(self.processid))
        if self.bk_out.poll() == "":
            warning(_("bashkaraoke_OUT not killed. Using «killall»."))
            self.killall()
        else:
            if not killed:
                success(l_killed + " " + str(self.processid))
        self.lyrics.set_label('''<span font="60">Bash!Karaoke
</span>\n<span font="20" weight="light">''' + self.l_btm + '''</span>''')
        self.playpause.set_icon_name("media-playback-start")
        f1 = open(user.home + "/.bashkaraoke/.lyrics", "w")
        f1.write("""<span font="50" weight="light">""" + l_loading + """</span>""")
        f1.close()

    def force_kill(self, force_stop):
        self.killall()

    def killall(self):
        global playing
        playing = False
        self.playpause.set_icon_name("media-playback-start")
        ucciditutti = subprocess.Popen("killall bash", shell=True)
        f1 = open(user.home + "/.bashkaraoke/.lyrics", "w")
        f1.write("""<span font="50" weight="light">""" + l_loading + """</span>""")
        f1.close()
        self.lyrics.set_label('''<span font="60" weight="light">Bash!Karaoke
        </span>\n<span font="20" weight="light">''' + self.l_btm + '''</span>''')
        success(_("bashkaraoke_OUT killed"))

    # Restore parameters
    def on_restore(self, restore):
        debug(_("Parameters restored"))
        self.pitch_adj.set_value(self.gtk_predefpitch)
        self.volume_adj.set_value(self.gtk_predefvolume)
        self.speed_adj.set_value(self.gtk_predefspeed)
        if not playing: pass
        elif playing :
            self.playpause.set_icon_name("dialog-ok")
        else:
            self.playpause.set_icon_name("dialog-ok")

    # Quit Bash!Karaoke GTK
    def destroy(self, main):
        if playing:
            self.stop()
        else: pass
        info(_("Thank you for using Bash!Karaoke"))
        log("", "close")
        Gtk.main_quit()
        sys.exit(0)

    def restart(self):
        if playing:
            self.stop()
        else: pass
        info(_("Thank you for using Bash!Karaoke"))
        log("", "restart")
        Gtk.main_quit()
        os.execl("/usr/bin/bashkaraoke-gtk", "")

#    def get_file_path_from_dnd_dropped_uri(uri):
#	    # get the path to file
#	    path = ""
#	    if uri.startswith('file:\\\\\\'): # windows
#	    	path = uri[8:] # 8 is len('file:///')
#	    elif uri.startswith('file://'): # nautilus, rox
#	    	path = uri[7:] # 7 is len('file://')
#	    elif uri.startswith('file:'): # xffm
#	    	path = uri[5:] # 5 is len('file:')
#
#	    path = urllib.url2pathname(path) # escape special chars
#	    path = path.strip('\r\n\x00') # remove \r\n and NULL
#
#	    return path
#
#    def on_drag_data_received(widget, context, x, y, selection, target_type, timestamp):
#	    if target_type == self.TARGET_TYPE_URI_LIST:
#	    	uri = selection.data.strip('\r\n\x00')
#	    	print 'uri', uri
#	    	uri_splitted = uri.split() # we may have more than one file dropped
#	    	for uri in uri_splitted:
#	    		path = get_file_path_from_dnd_dropped_uri(uri)
#	    		print 'path to open', path
#	    		if os.path.isfile(path): # is it file?
#	    			data = file(path).read()
#	    			#print data

    def fadein(self):
        self.main.set_opacity(0)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.01)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.02)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.03)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.04)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.05)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.06)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.07)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.08)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.09)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.1)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.11)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.12)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.13)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.14)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.15)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.16)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.17)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.18)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.19)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.2)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.21)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.22)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.23)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.24)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.25)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.26)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.27)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.28)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.29)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.3)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.31)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.32)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.33)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.34)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.35)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.36)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.37)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.38)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.39)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.4)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.41)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.42)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.43)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.44)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.45)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.46)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.47)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.48)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.49)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.5)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.51)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.52)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.53)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.54)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.55)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.56)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.57)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.58)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.59)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.6)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.61)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.62)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.63)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.64)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.65)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.66)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.67)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.68)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.69)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.7)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.71)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.72)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.73)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.74)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.75)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.76)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.77)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.78)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.79)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.8)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.81)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.82)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.83)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.84)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.85)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.86)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.87)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.88)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.89)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.9)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.91)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.92)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.93)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.94)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.95)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.96)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.97)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.98)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.99)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(1.0)
        while Gtk.events_pending():
            Gtk.main_iteration()

    def fadeout(self):
        self.main.set_opacity(1.0)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.99)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.98)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.97)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.96)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.95)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.94)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.93)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.92)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.91)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.9)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.89)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.88)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.87)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.86)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.85)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.84)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.83)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.82)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.81)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.8)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.79)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.78)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.77)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.76)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.75)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.74)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.73)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.72)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.71)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.7)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.69)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.68)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.67)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.66)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.65)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.64)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.63)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.62)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.61)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.6)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.59)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.58)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.57)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.56)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.55)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.54)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.53)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.52)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.51)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.5)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.49)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.48)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.47)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.46)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.45)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.44)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.43)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.42)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.41)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.4)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.39)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.38)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.37)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.36)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.35)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.34)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.33)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.32)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.31)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.3)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.29)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.28)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.27)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.26)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.25)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.24)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.23)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.22)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.21)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.2)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.19)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.18)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.17)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.16)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.15)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.14)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.13)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.12)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.11)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.1)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.09)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.08)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.07)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.06)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.05)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.04)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.03)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.02)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.01)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.main.set_opacity(0.00)
        while Gtk.events_pending():
            Gtk.main_iteration()

def mainshow():
    app = Main()
    Gtk.main()

if __name__ == "__main__":
    mainshow()

