#!/usr/bin/env python
# *-* coding: utf-8 *-*
#
# intro.py
# Copyright (C) Davide Depau 2011-2012 <david.dep.1996@gmail.com>
# 
# Bash!Karaoke GTK is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Bash!Karaoke GTK is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program.    If not, see <http://www.gnu.org/licenses/>.

# Import modules
from gi.repository import Gtk, Gdk, GdkPixbuf, GdkX11, GObject
import os, sys, pygst, user, time
pygst.require("0.10")
import gst

# Set Bash!Karaoke GTK's directory
bkdir = "/usr/share/bashkaraoke-gtk"

def exists(var):
    pass

# Enable/Disable debug mode
if os.path.exists(user.home + '/.bashkaraoke/.bashkaraoke.rc'):
    debugg = open(user.home + '/.bashkaraoke/.bashkaraoke.rc', 'r')
    for line in debugg.readlines():
        if "export show_debug=" in line:
            fooo = int(line.split("=", 1)[1])
            try: show_debug = bool(fooo)
            except: pass
            try: exists(show_debug)
            except: show_debug = False
    debugg.close()
else:
    show_debug = False

# Start logging debug messages
def log(logtext, logtype):
    log = open(user.home + "/.bashkaraoke/.bashkaraoke-gtk.log", "a")
    if logtype == "d":
        log.write("[" + time.asctime() + " DEBUG] " + str(logtext) + "\n")
        log.close()
    else:
        log.write("[" + time.asctime() + "] " + str(logtext) + "\n")
        log.close()

def debug(dmessage):
    if show_debug == True:
        print "[" + time.asctime() + " DEBUG] " + dmessage
    else: pass
    log(dmessage, "d")

# Main window
class Video:
    def __init__(self):
        # Set up UI
        self.win = Gtk.Window()
        self.win.set_title("Welcome to Bash!Karaoke 5.0.0")
        self.win.set_resizable(True)
        self.win.set_decorated(True)
        self.win.set_position(Gtk.WindowPosition.CENTER)
        self.win.connect("destroy", self.destroy)
        screen = self.win.get_screen()

        overlay = Gtk.Overlay()
        self.win.add(overlay)
        overlay.show()
        width, height = self.win.get_screen().get_width(), self.win.get_screen().get_height()
        self.videowidget = Gtk.DrawingArea()
        overlay.add(self.videowidget)
        self.videowidget.set_halign (Gtk.Align.START)
        self.videowidget.set_valign (Gtk.Align.START)
        self.videowidget.set_size_request(width, height)
        self.videowidget.show()

        fixed = Gtk.Fixed()

        fixed.set_halign(Gtk.Align.END)
        fixed.set_valign(Gtk.Align.END)

        overlay.add_overlay(fixed)
        fixed.show()

        button = Gtk.Button()
        button.set_label("Skip")
        fixed.put(button, 2, 2)
        button.show()
        button.connect("clicked", self.skip)

        # Choose best video resolution
        if width >= 1000:
            uri = bkdir + "/ui/intro1080i.ogg"
            debug("Video mode: 1080i (screen size: " + str(width) + "x" + str(height) + ")")
        else:
            uri = bkdir + "/ui/intro720p.ogg"
            debug("Video mode: 720 (default, screen size: " + str(width) + "x" + str(height) + ")")

        # Make window transparent and fullscreen, then fade it in
        self.win.set_opacity(0.0)
        self.win.show_all()
        self.win.fullscreen()
        self.fadein()

        # Setup GStreamer 
        self.player = gst.element_factory_make("playbin2", "player")
        bus = self.player.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission() 
        bus.connect("message", self.on_message)
        bus.connect("sync-message::element", self.on_sync_message)
        self.player.set_property("uri", "file://" + uri)
        self.player.set_state(gst.STATE_PLAYING)

    def on_sync_message(self, bus, message):
        if message.structure is None:
            return False
        if message.structure.get_name() == "prepare-xwindow-id":
            Gdk.threads_enter()
            Gdk.Display.get_default().sync()
            win_id = self.videowidget.get_property('window').get_xid()
            imagesink = message.src
            imagesink.set_property("force-aspect-ratio", True)
            imagesink.set_xwindow_id(win_id)
            Gdk.threads_leave()

    def on_message(self, bus, message):
        t = message.type
        if t == gst.MESSAGE_EOS:
            self.eos()
        elif t == gst.MESSAGE_ERROR:
            self.player.set_state(gst.STATE_NULL)
            err, debug = message.parse_error()
            sys.stderr.write(str("Error: %s" % err + debug))
            self.destroy()

    # Destroy window
    def destroy(self, win):
        self.player.set_state(gst.STATE_NULL)
        self.fadeout()
        Gtk.main_quit()

    # End Of Stream
    def eos(self):
        self.player.set_state(gst.STATE_NULL)
        self.fadeout()
        Gtk.main_quit()
        sys.exit()

    def skip(self, button):
        self.player.set_state(gst.STATE_NULL)
        self.fadeout()
        Gtk.main_quit()

    def fadeout(self):
        self.win.set_opacity(1.0)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.99)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.98)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.97)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.96)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.95)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.94)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.93)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.92)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.91)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.9)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.89)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.88)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.87)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.86)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.85)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.84)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.83)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.82)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.81)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.8)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.79)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.78)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.77)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.76)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.75)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.74)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.73)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.72)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.71)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.7)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.69)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.68)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.67)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.66)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.65)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.64)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.63)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.62)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.61)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.6)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.59)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.58)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.57)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.56)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.55)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.54)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.53)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.52)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.51)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.5)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.49)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.48)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.47)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.46)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.45)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.44)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.43)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.42)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.41)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.4)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.39)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.38)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.37)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.36)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.35)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.34)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.33)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.32)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.31)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.3)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.29)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.28)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.27)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.26)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.25)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.24)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.23)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.22)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.21)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.2)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.19)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.18)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.17)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.16)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.15)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.14)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.13)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.12)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.11)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.1)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.09)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.08)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.07)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.06)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.05)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.04)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.03)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.02)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.01)

    def fadein(self):
        self.win.set_opacity(0.0)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.01)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.02)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.03)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.04)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.05)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.06)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.07)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.08)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.09)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.1)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.11)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.12)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.13)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.14)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.15)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.16)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.17)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.18)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.19)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.2)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.21)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.22)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.23)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.24)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.25)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.26)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.27)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.28)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.29)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.3)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.31)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.32)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.33)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.34)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.35)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.36)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.37)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.38)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.39)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.4)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.41)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.42)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.43)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.44)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.45)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.46)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.47)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.48)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.49)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.5)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.51)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.52)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.53)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.54)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.55)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.56)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.57)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.58)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.59)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.6)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.61)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.62)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.63)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.64)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.65)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.66)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.67)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.68)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.69)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.7)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.71)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.72)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.73)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.74)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.75)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.76)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.77)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.78)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.79)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.8)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.81)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.82)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.83)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.84)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.85)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.86)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.87)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.88)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.89)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.9)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.91)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.92)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.93)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.94)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.95)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.96)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.97)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.98)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(0.99)
        time.sleep(0.006)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.win.set_opacity(1.00)


if __name__ == "__main__":
    Gdk.threads_init()
    Video()
    Gtk.main()
